@@ -10,7 +10,7 @@
 #******************************************************************************************
 #******************************************************************************************
 
-linarpr <- function(Y, id, weight=NULL, Y_thres=NULL, wght_thres=NULL,
+linarpr <- function(Y, id=NULL, weight=NULL, Y_thres=NULL, wght_thres=NULL,
                  sort=NULL, Dom=NULL, period=NULL, dataset = NULL, percentage=60, 
                  order_quant=50, var_name="lin_arpr") {
  
@@ -32,39 +32,31 @@ linarpr <- function(Y, id, weight=NULL, Y_thres=NULL, wght_thres=NULL,
       } else order_quant <- order_quant[1]
 
    if(!is.null(dataset)) {
-       dataset <- data.frame(dataset)
-       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y] 
+       dataset <- data.table(dataset)
+       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y, with=FALSE] 
 
        if(!is.null(id)) {
-          id2 <- id
-          if (checker(id, dataset, "id")) id <- data.frame(dataset[, id], stringsAsFactors=FALSE)
-          names(id) <- id2 }
+          if (checker(id, dataset, "id")) id <- dataset[, id, with=FALSE]}
 
        if(!is.null(weight)) {
-           if (checker(weight, dataset, "weight")) weight <- dataset[, weight] }
+           if (checker(weight, dataset, "weight")) weight <- dataset[, weight, with=FALSE] }
 
        if(!is.null(Y_thres)) {
-           if (checker(Y_thres, dataset, "Y_thres")) Y_thres <- dataset[, Y_thres] }
+           if (checker(Y_thres, dataset, "Y_thres")) Y_thres <- dataset[, Y_thres, with=FALSE] }
      
        if(!is.null(wght_thres)) {
-           if (checker(wght_thres, dataset, "wght_thres")) wght_thres <- dataset[, wght_thres] }
+           if (checker(wght_thres, dataset, "wght_thres")) wght_thres <- dataset[, wght_thres, with=FALSE] }
 
        if(!is.null(sort)) {
-           if (checker(sort, dataset, "sort")) sort <- dataset[, sort] }
+           if (checker(sort, dataset, "sort")) sort <- dataset[, sort, with=FALSE] }
 
        if (!is.null(period)) {
-            aperiod <- period  
             if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
-            if (min(period %in% names(dataset))==1) {
-                                period <- data.frame(dataset[, period], stringsAsFactors=FALSE)
-                                names(period) <- aperiod }}
+            if (min(period %in% names(dataset))==1) period <- dataset[, period, with=FALSE]}
 
        if (!is.null(Dom)) {
-            Dom2 <- Dom
-            if (checker(Dom, dataset, "Dom")) {
-                    Dom <- data.table(dataset[, Dom]) 
-                    setnames(Dom, names(Dom), Dom2) }    }
-      }
+            if (checker(Dom, dataset, "Dom")) Dom <- data.table(dataset[, Dom, with=FALSE])}
+    }
 
    # check vectors
    # Y
@@ -104,9 +96,12 @@ linarpr <- function(Y, id, weight=NULL, Y_thres=NULL, wght_thres=NULL,
    if (any(is.na(wght_thres))) stop("'wght_thres' has unknown values") 
     
    # sort
-   if (!is.null(sort) && !is.vector(sort) && !is.ordered(sort)) {
-         stop("'sort' must be a vector or ordered factor") }
-   if (!is.null(sort) && length(sort) != n) stop("'sort' must be the same length as 'Y'")  
+   if (!is.null(sort)) {    
+        sort <- data.frame(sort)
+        if (length(sort) != n) stop("'sort' must have the same length as 'Y'")
+        if (ncol(sort) != 1) stop("'sort' must be vector or 1 column data.frame, matrix, data.table")
+        sort <- sort[, 1]
+   }
 
    # period     
    if (!is.null(period)) {
@@ -115,7 +110,7 @@ linarpr <- function(Y, id, weight=NULL, Y_thres=NULL, wght_thres=NULL,
                  stop("'period' are duplicate column names: ", 
                       paste(names(period)[duplicated(names(period))], collapse = ","))
        if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
-       if(any(is.na(period))) stop("'period' has unknown values")  
+       if(any(is.na(period))) stop("'period' has unknown values")
    }   
    
    # id
@@ -139,7 +134,7 @@ linarpr <- function(Y, id, weight=NULL, Y_thres=NULL, wght_thres=NULL,
                       paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
              if (is.null(names(Dom))) stop("'Dom' must be colnames")
              if (nrow(Dom) != n) stop("'Dom' must be the same length as 'Y'")
-             Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
+             Dom[, (names(Dom)):=lapply(.SD, as.character)] 
        }
 
     ## computations
