@@ -10,7 +10,7 @@
 #******************************************************************************************
 #******************************************************************************************
 
-linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=NULL, 
+linarr <- function(Y, Y_den, id=NULL, age, pl085, month_at_work, weight=NULL,  sort=NULL, 
                              Dom=NULL, period=NULL, dataset = NULL, 
                              order_quant=50, var_name="lin_arr") {
  
@@ -26,43 +26,36 @@ linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=N
       } else order_quant <- order_quant[1]
 
    if(!is.null(dataset)) {
-       dataset <- data.frame(dataset)
-       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y] 
-       if (checker(Y_den, dataset, "Y_den"))  Y_den <- dataset[, Y_den] 
+       dataset <- data.table(dataset)
+       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y, with=FALSE] 
+       if (checker(Y_den, dataset, "Y_den"))  Y_den <- dataset[, Y_den, with=FALSE] 
 
        if(!is.null(id)) {
-          id2 <- id
-          if (checker(id, dataset, "id")) id <- data.frame(dataset[, id], stringsAsFactors=FALSE)
-          names(id) <- id2 }
+          if (checker(id, dataset, "id")) id <- dataset[, id, with=FALSE] }
 
        if(!is.null(age)) {
-           if (checker(age, dataset, "age")) age <- dataset[, age] }
+           if (checker(age, dataset, "age")) age <- dataset[, age, with=FALSE] }
 
        if(!is.null(pl085)) {
-           if (checker(pl085, dataset, "pl085")) pl085 <- dataset[, pl085] }
+           if (checker(pl085, dataset, "pl085")) pl085 <- dataset[, pl085, with=FALSE] }
 
        if(!is.null(month_at_work)) {
-           if (checker(month_at_work, dataset, "month_at_work")) month_at_work <- dataset[, month_at_work] }
+           if (checker(month_at_work, dataset, "month_at_work")) {
+                  month_at_work <- dataset[, month_at_work, with=FALSE] }}
 
        if(!is.null(weight)) {
-           if (checker(weight, dataset, "weight")) weight <- dataset[, weight] }
+           if (checker(weight, dataset, "weight")) weight <- dataset[, weight, with=FALSE] }
 
        if(!is.null(sort)) {
-           if (checker(sort, dataset, "sort")) sort <- dataset[, sort] }
+           if (checker(sort, dataset, "sort")) sort <- dataset[, sort, with=FALSE] }
 
        if (!is.null(period)) {
-            aperiod <- period  
             if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
-            if (min(period %in% names(dataset))==1) {
-                                period <- data.frame(dataset[, period], stringsAsFactors=FALSE)
-                                names(period) <- aperiod }}
+            if (min(period %in% names(dataset))==1) period <- dataset[, period, with=FALSE] }
 
        if (!is.null(Dom)) {
-            Dom2 <- Dom
-            if (checker(Dom, dataset, "Dom")) {
-                    Dom <- data.table(dataset[, Dom]) 
-                    setnames(Dom, names(Dom), Dom2) }    }
-      }
+            if (checker(Dom, dataset, "Dom")) Dom <- dataset[, Dom, with=FALSE] }
+    }
 
    # check vectors
    # Y
@@ -113,9 +106,12 @@ linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=N
    if (any(is.na(pl085))) stop("'month_at_work' has unknown values")
 
    # sort
-   if (!is.null(sort) && !is.vector(sort) && !is.ordered(sort)) {
-         stop("'sort' must be a vector or ordered factor") }
-   if (!is.null(sort) && length(sort) != n) stop("'sort' must be the same length as 'Y'")  
+   if (!is.null(sort)) {    
+        sort <- data.frame(sort)
+        if (length(sort) != n) stop("'sort' must have the same length as 'Y'")
+        if (ncol(sort) != 1) stop("'sort' must be vector or 1 column data.frame, matrix, data.table")
+        sort <- sort[, 1]
+   }
 
    # period     
    if (!is.null(period)) {
@@ -124,7 +120,7 @@ linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=N
                  stop("'period' are duplicate column names: ", 
                       paste(names(period)[duplicated(names(period))], collapse = ","))
        if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
-       if(any(is.na(period))) stop("'period' has unknown values")  
+       if(any(is.na(period))) stop("'period' has unknown values")
    }   
    
    # id
@@ -148,7 +144,7 @@ linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=N
                       paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
              if (is.null(names(Dom))) stop("'Dom' must be colnames")
              if (nrow(Dom) != n) stop("'Dom' must be the same length as 'Y'")
-             Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
+             Dom[, (names(Dom)):=lapply(.SD, as.character)]
        }
 
     ## computations
@@ -207,12 +203,12 @@ linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=N
               ind <- as.integer(rowSums(Dom == Dom_agg[i,][ind0,]) == ncol(Dom))
 
               arrl <- lapply(1:nrow(period1_agg), function(j) {
-                               if (!is.null(period)) { 
+                              if (!is.null(period)) { 
                                        rown <- cbind(period_agg[j], Dom_agg[i])
                                        setkeyv(rown, names(rown))
                                        rown2 <- copy(rown)
                                        rown <- merge(rown, quantile, all.x=TRUE)
-                                     } else {rown <- quantile
+                                     } else {rown <- quantile[i]
                                              rown2 <- Dom_agg[i] }
 
                                indj <- (rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))
@@ -239,13 +235,12 @@ linarr <- function(Y, Y_den, id, age, pl085, month_at_work, weight=NULL,  sort=N
                  arr_m <- merge(arr_m, arrlin, all.x=TRUE)
                  arr_v <- rbind(arr_v, arrs) 
            }
-     } else { arrl <- lapply(1:nrow(period1_agg), function(j) {
-j=1              
-             if (!is.null(period)) { 
-                                         rown <- period_agg[j]
-                                         setkeyv(rown, names(rown))
-                                         rown <- merge(rown, quantile, all.x=TRUE)
-                                       } else rown <- quantile
+     } else { arrl <- lapply(1:nrow(period1_agg), function(j) {  
+         
+             if (!is.null(period)) { rown <- period_agg[j]
+                                     setkeyv(rown, names(rown))
+                                     rown <- merge(rown, quantile, all.x=TRUE)
+                                 } else rown <- quantile
                            ind2 <- (rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))
       
                            arr_l <- arrlinCalc(Y_num=Y[ind2],
