@@ -10,9 +10,9 @@
 #******************************************************************************************
 #******************************************************************************************
 
-linrmir <- function(Y, id, age, weight=NULL,  sort=NULL, 
-                             Dom=NULL, period=NULL, dataset = NULL, 
-                             order_quant=50, var_name="lin_rmir") {
+linrmir <- function(Y, id=NULL, age, weight=NULL, sort=NULL, 
+                              Dom=NULL, period=NULL, dataset = NULL, 
+                              order_quant=50, var_name="lin_rmir") {
  
    ## initializations
    if (min(dim(data.table(var_name))==1)!=1) {
@@ -26,35 +26,27 @@ linrmir <- function(Y, id, age, weight=NULL,  sort=NULL,
       } else order_quant <- order_quant[1]
 
    if(!is.null(dataset)) {
-       dataset <- data.frame(dataset)
-       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y] 
+       dataset <- data.table(dataset)
+       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y, with=FALSE] 
 
        if(!is.null(id)) {
-          id2 <- id
-          if (checker(id, dataset, "id")) id <- data.frame(dataset[, id], stringsAsFactors=FALSE)
-          names(id) <- id2 }
+          if (checker(id, dataset, "id")) id <- dataset[, id, with=FALSE] }
 
        if(!is.null(age)) {
-           if (checker(age, dataset, "age")) age <- dataset[, age] }
+           if (checker(age, dataset, "age")) age <- dataset[, age, with=FALSE] }
 
        if(!is.null(weight)) {
-           if (checker(weight, dataset, "weight")) weight <- dataset[, weight] }
+           if (checker(weight, dataset, "weight")) weight <- dataset[, weight, with=FALSE] }
 
        if(!is.null(sort)) {
-           if (checker(sort, dataset, "sort")) sort <- dataset[, sort] }
+           if (checker(sort, dataset, "sort")) sort <- dataset[, sort, with=FALSE] }
 
        if (!is.null(period)) {
-            aperiod <- period  
             if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
-            if (min(period %in% names(dataset))==1) {
-                                period <- data.frame(dataset[, period], stringsAsFactors=FALSE)
-                                names(period) <- aperiod }}
+            if (min(period %in% names(dataset))==1) period <- dataset[, period, with=FALSE] }
 
        if (!is.null(Dom)) {
-            Dom2 <- Dom
-            if (checker(Dom, dataset, "Dom")) {
-                    Dom <- data.table(dataset[, Dom]) 
-                    setnames(Dom, names(Dom), Dom2) }    }
+            if (checker(Dom, dataset, "Dom")) Dom <- dataset[, Dom, with=FALSE] }
       }
 
    # check vectors
@@ -83,9 +75,12 @@ linrmir <- function(Y, id, age, weight=NULL,  sort=NULL,
    if (any(is.na(age))) stop("'age' has unknown values")
    
    # sort
-   if (!is.null(sort) && !is.vector(sort) && !is.ordered(sort)) {
-         stop("'sort' must be a vector or ordered factor") }
-   if (!is.null(sort) && length(sort) != n) stop("'sort' must be the same length as 'Y'")  
+   if (!is.null(sort)) {
+        sort <- data.frame(sort)
+        if (length(sort) != n) stop("'sort' must have the same length as 'Y'")
+        if (ncol(sort) != 1) stop("'sort' must be vector or 1 column data.frame, matrix, data.table")
+        sort <- sort[, 1]
+   }
 
    # period     
    if (!is.null(period)) {
@@ -94,7 +89,7 @@ linrmir <- function(Y, id, age, weight=NULL,  sort=NULL,
                  stop("'period' are duplicate column names: ", 
                       paste(names(period)[duplicated(names(period))], collapse = ","))
        if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
-       if(any(is.na(period))) stop("'period' has unknown values")  
+       if(any(is.na(period))) stop("'period' has unknown values")
    }   
    
    # id
@@ -118,7 +113,7 @@ linrmir <- function(Y, id, age, weight=NULL,  sort=NULL,
                       paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
              if (is.null(names(Dom))) stop("'Dom' must be colnames")
              if (nrow(Dom) != n) stop("'Dom' must be the same length as 'Y'")
-             Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
+             Dom[, (names(Dom)):=lapply(.SD, as.character)]
        }
 
     ## computations
@@ -160,17 +155,19 @@ linrmir <- function(Y, id, age, weight=NULL,  sort=NULL,
         rmir_v <- c()
         rmir_m <- copy(rmir_id)
         for(i in 1:nrow(Dom_agg)) {
+
               g <- c(var_name, paste(names(Dom), as.matrix(Dom_agg[i,]), sep = "."))
               var_nams <- do.call(paste, as.list(c(g, sep="__")))
               ind <- as.integer(rowSums(Dom == Dom_agg[i,][ind0,]) == ncol(Dom))
 
               rmirl <- lapply(1:nrow(period1_agg), function(j) {
+
                                if (!is.null(period)) { 
                                        rown <- cbind(period_agg[j], Dom_agg[i])
                                        setkeyv(rown, names(rown))
                                        rown2 <- copy(rown)
                                        rown <- merge(rown, quantile, all.x=TRUE)
-                                     } else {rown <- quantile
+                                     } else {rown <- quantile[i]
                                              rown2 <- Dom_agg[i] }
 
                                indj <- (rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))
