@@ -10,7 +10,7 @@
 #*********************************************************************************************************
 #*********************************************************************************************************
 
-linrmpg <- function(Y, id, weight=NULL, sort=NULL, Dom=NULL,
+linrmpg <- function(Y, id=NULL, weight=NULL, sort=NULL, Dom=NULL,
                     period=NULL, dataset = NULL, percentage=60, 
                     order_quant=50, var_name="lin_rmpg") {
  
@@ -33,31 +33,24 @@ linrmpg <- function(Y, id, weight=NULL, sort=NULL, Dom=NULL,
       } else order_quant <- order_quant[1]
 
    if (!is.null(dataset)) {
-        dataset <- data.frame(dataset)
-        if (checker(Y, dataset, "Y")) Y <- dataset[, Y] 
+        dataset <- data.table(dataset)
+        if (checker(Y, dataset, "Y")) Y <- dataset[, Y, with=FALSE] 
 
         if (!is.null(id)) {
-             id2 <- id
-             if (checker(id, dataset, "id")) id <- data.frame(dataset[, id], stringsAsFactors=FALSE)
-             names(id) <- id2 }
+             if (checker(id, dataset, "id")) id <- dataset[, id, with=FALSE] }
 
         if(!is.null(weight)) {
-            if (checker(weight, dataset, "weight")) weight <- dataset[, weight] }
+            if (checker(weight, dataset, "weight")) weight <- dataset[, weight, with=FALSE] }
 
         if(!is.null(sort)) {
-            if (checker(sort, dataset, "sort")) sort <- dataset[, sort] }
+            if (checker(sort, dataset, "sort")) sort <- dataset[, sort, with=FALSE] }
 
        if (!is.null(period)) {
-            aperiod <- period  
             if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
-            if (min(period %in% names(dataset))==1) {
-                                period <- data.frame(dataset[, period], stringsAsFactors=FALSE)
-                                names(period) <- aperiod }}
+            if (min(period %in% names(dataset))==1) period <- dataset[, period, with=FALSE] }
+
         if (!is.null(Dom)) {
-             Dom2<-Dom
-             if (checker(Dom, dataset, "Dom")) {
-                     Dom <- as.data.frame(dataset[, Dom], stringsAsFactors=FALSE) 
-                     names(Dom) <- Dom2 }    }
+             if (checker(Dom, dataset, "Dom")) Dom <- dataset[, Dom, with=FALSE] }
       }
 
    # check vectors
@@ -78,10 +71,13 @@ linrmpg <- function(Y, id, weight=NULL, sort=NULL, Dom=NULL,
    if (!is.numeric(weight)) stop("'weight' must be numerical")
    if (any(is.na(weight))) stop("'weight' has unknown values")
 
-   # sort
-   if (!is.null(sort) && !is.vector(sort) && !is.ordered(sort)) {
-         stop("'sort' must be a vector or ordered factor") }
-   if (!is.null(sort) && length(sort) != n) stop("'sort' must be the same length as 'Y'")     
+   # sort  
+   if (!is.null(sort)) {
+        sort <- data.frame(sort)
+        if (length(sort) != n) stop("'sort' must have the same length as 'Y'")
+        if (ncol(sort) != 1) stop("'sort' must be vector or 1 column data.frame, matrix, data.table")
+        sort <- sort[, 1]
+   }
    
    # period     
    if (!is.null(period)) {
@@ -89,8 +85,8 @@ linrmpg <- function(Y, id, weight=NULL, sort=NULL, Dom=NULL,
        if (any(duplicated(names(period)))) 
                  stop("'period' are duplicate column names: ", 
                       paste(names(period)[duplicated(names(period))], collapse = ","))
-       if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
-       if(any(is.na(period))) stop("'period' has unknown values")  
+       if (nrow(period) != n) stop("'period' must be th e same length as 'Y'")
+       if(any(is.na(period))) stop("'period' has unknown values")
    }   
 
    # id
@@ -104,6 +100,7 @@ linrmpg <- function(Y, id, weight=NULL, sort=NULL, Dom=NULL,
                        } else {
                           id1 <- data.table(period, id)
                           if (any(duplicated(id1))) stop("'id' by period are duplicate values")
+                          id1 <- NULL
                          }
 
    # Dom     
@@ -114,9 +111,9 @@ linrmpg <- function(Y, id, weight=NULL, sort=NULL, Dom=NULL,
                       paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
              if (is.null(names(Dom))) stop("'Dom' must be colnames")
              if (nrow(Dom) != n) stop("'Dom' must be the same length as 'Y'")
-            Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
+             Dom[, (names(Dom)):=lapply(.SD, as.character)]  
        }
-
+ 
     ## computations
     ind0 <- rep.int(1, n)
     period_agg <- period1 <- NULL
