@@ -24,9 +24,8 @@ vardom <- function(Y, H, PSU, w_final,
                    X = NULL,
                    ind_gr = NULL,
                    g = NULL,
+                   q = NULL,
                    dataset = NULL, 
-                   q = rep(1, if (is.null(dataset)) 
-                           nrow(data.frame(X)) else nrow(dataset)),
                    confidence = .95,
                    outp_lin = FALSE,
                    outp_res = FALSE) {
@@ -39,71 +38,43 @@ vardom <- function(Y, H, PSU, w_final,
           stop("'confidence' must be a numeric value in [0,1]")  }
 
   if(!is.null(dataset)) {
-      dataset <- data.frame(dataset)
-      aY <- Y
+      dataset <- data.table(dataset)
       if (min(Y %in% names(dataset))!=1) stop("'Y' does not exist in 'dataset'!")
-      if (min(Y %in% names(dataset))==1) {
-                                Y <- data.frame(dataset[, Y], check.names=FALSE)
-                                names(Y) <- aY }
+      if (min(Y %in% names(dataset))==1) { Y <- dataset[, Y, with=FALSE] }
       if(!is.null(id)) {
-          id2 <- id
           if (min(id %in% names(dataset))!=1) stop("'id' does not exist in 'dataset'!")
-          if (min(id %in% names(dataset))==1) id <- data.frame(dataset[, id])
-          names(id) <- id2  }
+          if (min(id %in% names(dataset))==1) id <- dataset[, id, with=FALSE] }
       if(!is.null(H)) {
-          aH <- H  
           if (min(H %in% names(dataset))!=1) stop("'H' does not exist in 'dataset'!")
-          if (min(H %in% names(dataset))==1) {
-                                H <- as.data.frame(dataset[, aH], stringsAsFactors=FALSE)
-                                names(H) <- aH }}
+          if (min(H %in% names(dataset))==1) H <- dataset[, H, with=FALSE] }
       if(!is.null(PSU)) {
-          aPSU <- PSU  
           if (min(PSU %in% names(dataset))!=1) stop("'PSU' does not exist in 'dataset'!")
-          if (min(PSU %in% names(dataset))==1) {
-                                PSU <- as.data.frame(dataset[, aPSU], stringsAsFactors=FALSE)
-                                names(PSU) <- aPSU }}
+          if (min(PSU %in% names(dataset))==1) PSU <- dataset[, PSU, with=FALSE] }
       if(!is.null(w_final)) {
-          aw_final <- w_final  
           if (min(w_final %in% names(dataset))!=1) stop("'w_final' does not exist in 'dataset'!")
-          if (min(w_final %in% names(dataset))==1) {
-                                w_final <- data.frame(dataset[, aw_final])
-                                names(w_final) <- aw_final }}
+          if (min(w_final %in% names(dataset))==1) w_final <- dataset[, w_final, with=FALSE] }
       if(!is.null(Z)) {
-          aZ <- Z
           if (min(Z %in% names(dataset))!=1) stop("'Z' does not exist in 'dataset'!")
-          if (min(Z %in% names(dataset))==1) {
-                                Z <-data.frame(dataset[, aZ],check.names=FALSE, stringsAsFactors=FALSE)
-                                names(Z) <- aZ }}
+          if (min(Z %in% names(dataset))==1) Z <- dataset[, Z, with=FALSE] }
       if (!is.null(period)) {
-           aperiod <- period  
            if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
-           if (min(period %in% names(dataset))==1) {
-                               period <- as.data.frame(dataset[, aperiod], stringsAsFactors=FALSE)
-                               names(period) <- aperiod }} 
+           if (min(period %in% names(dataset))==1) period <- dataset[, period, with=FALSE] } 
       if(!is.null(X)) {
           if (min(X %in% names(dataset))!=1) stop("'X' does not exist in 'dataset'!")
-          if (min(X %in% names(dataset))==1) X <- dataset[, X] }
-
+          if (min(X %in% names(dataset))==1) X <- dataset[, X, with=FALSE] }
       if(!is.null(ind_gr)) {
           if (min(ind_gr %in% names(dataset))!=1) stop("'ind_gr' does not exist in 'dataset'!")
-          if (min(ind_gr %in% names(dataset))==1) ind_gr <- dataset[, ind_gr] } 
-
+          if (min(ind_gr %in% names(dataset))==1) ind_gr <- dataset[, ind_gr, with=FALSE] } 
       if(!is.null(g)) {
           if (min(g %in% names(dataset))!=1) stop("'g' does not exist in 'dataset'!")
-          if (min(g %in% names(dataset))==1) g <- dataset[, g] }
-     
+          if (min(g %in% names(dataset))==1) g <- dataset[, g, with=FALSE] }
       if(!is.null(q)) {
-          if (min(q %in% names(dataset))!=1) {
-              if (length(q)!=nrow(dataset))  stop("'q' does not exist in 'dataset'!") }
-          if (min(q %in% names(dataset))==1) q <- dataset[, q] } 
-     
+          if (min(q %in% names(dataset))!=1) stop("'q' does not exist in 'dataset'!") 
+          if (min(q %in% names(dataset))==1) q <- dataset[, q, with=FALSE] } 
       if (!is.null(Dom)) {
-          Dom2 <- Dom
-          if (min(Dom %in% names(dataset))!=1) stop("'Dom' does not exist in 'data'!")
-          if (min(Dom %in% names(dataset))==1) {  
-                  Dom <- as.data.frame(dataset[, Dom2], stringsAsFactors=FALSE) 
-                  names(Dom) <- Dom2 }    }
-      }
+          if (min(Dom %in% names(dataset))!=1) stop("'Dom' does not exist in 'dataset'!")
+          if (min(Dom %in% names(dataset))==1) Dom <- dataset[, Dom, with=FALSE] }   
+     }
 
   # Y
   Y <- data.table(Y, check.names=TRUE)
@@ -119,13 +90,15 @@ vardom <- function(Y, H, PSU, w_final,
   if (ncol(H) != 1) stop("'H' must be 1 column data.frame, matrix, data.table")
   if (any(is.na(H))) stop("'H' has unknown values")
   if (is.null(names(H))) stop("'H' must be colnames")
+  H[, (names(H)):=lapply(.SD, as.character)]
   
   # PSU
   PSU <- data.table(PSU)
   if (any(is.na(PSU))) stop("'PSU' has unknown values")
   if (nrow(PSU) != n) stop("'PSU' length must be equal with 'Y' row count")
   if (ncol(PSU) != 1) stop("'PSU' has more than 1 column")
-  
+  PSU[, (names(PSU)):=lapply(.SD, as.character)]
+
   # w_final 
   w_final <- data.frame(w_final)
   if (nrow(w_final) != n) stop("'w_final' must be equal with 'Y' row count")
@@ -162,22 +135,19 @@ vardom <- function(Y, H, PSU, w_final,
       if (!is.numeric(N_h[[ncol(N_h)]])) stop("The last column of 'N_h' should be numerical")
       if (any(is.na(N_h))) stop("'N_h' has unknown values") 
       if (is.null(names(N_h))) stop("'N_h' must be colnames")
-      if (is.null(names(N_h))) stop("'N_h' must be colnames")
-      namesH <- names(H)
-      if (H[, class(get(namesH))]!=N_h[, class(get(namesH))]) 
-                                         stop("Strata class for 'H' and 'N_h' is not equal ")
-
+      if (all(names(H) %in% names(N_h))) {N_h[, (names(H)):=lapply(.SD, as.character), .SDcols=names(H)]
+             } else stop("All strata titles of 'H' have not in 'N_h'")
       if (is.null(period)) {
              if (names(H) != names(N_h)[1]) stop("Strata titles for 'H' and 'N_h' is not equal")
-             if (any(is.na(merge(unique(H), N_h, by=names(H), all.x = T)))) stop("'N_h' is not defined for all stratas")
+             if (any(is.na(merge(unique(H), N_h, by=names(H), all.x = TRUE)))) stop("'N_h' is not defined for all stratas")
              if (any(duplicated(N_h[, head(names(N_h),-1), with=F]))) stop("Strata values for 'N_h' must be unique")
        } else { pH <- data.table(period, H)
                 if (any(names(pH) != names(N_h)[c(1:(1+np))])) stop("Strata titles for 'period' with 'H' and 'N_h' is not equal")
                 nperH <- names(period)
                 if (pH[, class(get(nperH))]!=N_h[, class(get(nperH))]) 
                                                        stop("Period class for 'period' and 'N_h' is not equal ")
-                if (any(is.na(merge(unique(pH), N_h, by=names(pH), all.x = T)))) stop("'N_h' is not defined for all stratas and periods")
-                if (any(duplicated(N_h[, head(names(N_h),-1), with=F]))) stop("Strata values for 'N_h' must be unique in all periods")
+                if (any(is.na(merge(unique(pH), N_h, by=names(pH), all.x=TRUE)))) stop("'N_h' is not defined for all stratas and periods")
+                if (any(duplicated(N_h[, head(names(N_h),-1), with=FALSE]))) stop("Strata values for 'N_h' must be unique in all periods")
                 pH <- NULL
      }
     setkeyv(N_h, names(N_h)[c(1:(1+np))])
@@ -187,14 +157,14 @@ vardom <- function(Y, H, PSU, w_final,
   namesDom <- NULL  
   if (!is.null(Dom)) {
     Dom <- data.table(Dom)
-    if (any(duplicated(names(Dom)))) 
+    namesDom <- names(Dom)
+    if (any(duplicated(namesDom))) 
            stop("'Dom' are duplicate column names: ", 
                  paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
     if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
     if (any(is.na(Dom))) stop("'Dom' has unknown values")
-    if (is.null(names(Dom))) stop("'Dom' must be colnames")
-    Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
-    namesDom <- names(Dom)
+    if (is.null(namesDom)) stop("'Dom' must be colnames")
+    Dom[, (namesDom):=lapply(.SD, as.character)]
   }
   
   # Z
@@ -251,6 +221,7 @@ vardom <- function(Y, H, PSU, w_final,
     
   # q
   if (!is.null(X)) {
+    if (is.null(q))  q <- rep(1, nrow(X))
     if (is.null(class(q)) | all(class(q)=="function")) stop("'q' must be numerical")
     q <- data.frame(q)
     if (nrow(q) != nrow(X)) stop("'q' length must be equal with 'X' row count")
